"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.PcxParser = void 0;
var FileParser_1 = require("./FileParser");
var PALETTE_SIZE = 769;
var PcxParser = (function (_super) {
    __extends(PcxParser, _super);
    function PcxParser(buffer) {
        var _this = _super.call(this, buffer) || this;
        _this.header = _this.readHeader();
        console.log(_this.header);
        _this.validateHeader();
        _this.width = _this.header.xEnd - _this.header.xStart;
        _this.height = _this.header.yEnd - _this.header.yStart;
        _this.palette = _this.readPalette();
        _this.pixels = _this.readData();
        return _this;
    }
    PcxParser.prototype.getPixels = function () {
        return this.pixels;
    };
    PcxParser.prototype.readHeader = function () {
        return {
            identifier: this.readInt(1),
            version: this.readInt(1),
            encoding: this.readInt(1),
            bitsPerPixel: this.readInt(1),
            xStart: this.readInt(2, false),
            yStart: this.readInt(2, false),
            xEnd: this.readInt(2, false),
            yEnd: this.readInt(2, false),
            hRes: this.readInt(2, false),
            vRes: this.readInt(2, false),
            palette16: this.readArray(48),
            reserved1: this.readInt(1),
            numBitPlanes: this.readInt(1),
            bytesPerLine: this.readInt(2, false),
            paletteType: this.readInt(2, false),
            hSize: this.readInt(2, false),
            vSize: this.readInt(2, false),
            reserved2: this.readInt(54),
        };
    };
    PcxParser.prototype.validateHeader = function () {
        if (this.header.identifier !== 0x0a) {
            throw new Error("Invalid pcx file");
        }
        if (![0, 2, 3, 4, 5].includes(this.header.version)) {
            throw new Error("Invalid version");
        }
        if (![0, 1].includes(this.header.encoding)) {
            throw new Error("Invalid encoding");
        }
        if (this.header.encoding === 0) {
            throw new Error("Does not support un-encoded files");
        }
        if (![1, 2, 4, 8].includes(this.header.bitsPerPixel)) {
            throw new Error("BitsPerPixel incorrect");
        }
        if (![1, 2].includes(this.header.paletteType)) {
            throw new Error("Invalid pallete mode");
        }
    };
    PcxParser.prototype.readPalette = function () {
        this.pushPointer();
        this.movePointer(this.getLength() - PALETTE_SIZE);
        var palleteId = this.readInt(1);
        if (palleteId !== 12) {
            this.popPointer();
            return this.header.palette16;
        }
        var arr = this.readArray(PALETTE_SIZE - 1);
        this.popPointer();
        return arr;
    };
    PcxParser.prototype.readData = function () {
        var scanLineLength = this.header.numBitPlanes * this.header.bytesPerLine;
        var buffer = new Uint8Array(scanLineLength * this.height);
        var index = 0;
        for (var t = 0; t < this.height; t += 1) {
            do {
                var byte = this.readInt(1);
                var runCount = 1;
                var runValue = byte;
                if ((byte & 0xc0) === 0xc0) {
                    runCount = byte & 0x3f;
                    runValue = this.readInt(1);
                }
                for (var i = 0; i < runCount; i += 1) {
                    buffer[index] = runValue;
                    index += 1;
                }
            } while (index < scanLineLength);
        }
        var pixels = new Uint8Array(this.width * this.height * 4);
        if (this.header.numBitPlanes === 1) {
            for (var bufferPos = 0, x = 0; x < pixels.length; bufferPos += 1) {
                var bufVal = buffer[bufferPos];
                if (typeof bufVal === "undefined") {
                    throw new Error("Buffer out of bounds");
                }
                if (this.header.bitsPerPixel === 1) {
                    for (var binIndex = 0; binIndex < 8; binIndex += 1) {
                        var color = ((bufVal >> binIndex) & 1) * 255;
                        pixels[x] = color;
                        pixels[x + 1] = color;
                        pixels[x + 2] = color;
                        pixels[x + 3] = 255;
                        x += 4;
                    }
                }
                else if (this.header.bitsPerPixel === 8) {
                    console.log(bufVal);
                    var color = this.getPalValues(bufVal);
                    pixels[x] = color.r;
                    pixels[x + 1] = color.g;
                    pixels[x + 2] = color.b;
                    pixels[x + 3] = color.a;
                    x += 4;
                }
            }
        }
        return pixels;
    };
    PcxParser.prototype.getPalValues = function (index) {
        var r = 0;
        var g = 0;
        var b = 0;
        var a = 255;
        if (this.header.bitsPerPixel === 2) {
            r = this.palette[index * 3];
            g = this.palette[index * 3 + 1];
            b = this.palette[index * 3 + 2];
            a = 255;
        }
        else if (this.header.bitsPerPixel === 4) {
            r = this.palette[index * 3];
            g = this.palette[index * 3 + 1];
            b = this.palette[index * 3 + 2];
            a = 255;
        }
        else if (this.header.bitsPerPixel === 8) {
            r = this.palette[index * 3];
            g = this.palette[index * 3 + 1];
            b = this.palette[index * 3 + 2];
            a = 255;
        }
        if (typeof r === "undefined" ||
            typeof g === "undefined" ||
            typeof b === "undefined" ||
            typeof a === "undefined") {
            throw new Error("Palette index out of bounds");
        }
        return { r: r, g: g, b: b, a: a };
    };
    return PcxParser;
}(FileParser_1.FileParser));
exports.PcxParser = PcxParser;
//# sourceMappingURL=PcxParser.js.map