"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadpng_init = exports.set_color_conversion = exports.register_bitmap_file_type = exports.save_tga_pf = exports.save_tga = exports.save_pcx_pf = exports.save_pcx = exports.save_bmp_pf = exports.save_bmp = exports.save_bitmap = exports.load_tga_pf = exports.load_tga = exports.load_pcx_pf = exports.load_pcx = exports.load_lbm = exports.load_bmp_pf = exports.load_bmp = exports.load_bitmap = exports.is_inside_bitmap = exports.get_clip_state = exports.set_clip_state = exports.add_clip_rect = exports.get_clip_rect = exports.set_clip_rect = exports.release_screen = exports.acquire_screen = exports.release_bitmap = exports.acquire_bitmap = exports.is_sub_bitmap = exports.is_system_bitmap = exports.is_video_bitmap = exports.is_screen_bitmap = exports.is_memory_bitmap = exports.is_linear_bitmap = exports.is_planar_bitmap = exports.is_same_bitmap = exports.bitmap_mask_color = exports.bitmap_color_depth = exports.lock_bitmap = exports.destroy_bitmap = exports.create_system_bitmap = exports.create_video_bitmap = exports.create_sub_bitmap = exports.create_bitmap_ex = exports.create_bitmap = exports.screen = void 0;
var color_1 = require("./color");
var core_1 = require("./core");
var debug_1 = require("./debug");
var sprites_1 = require("./sprites");
exports.screen = {
    w: 0,
    h: 0,
    canvas: null,
    context: null,
    ready: false,
    type: "bmp",
    clipping: false,
    clipping_rect: { x1: 0, y1: 0, x2: 0, y2: 0 },
    is_screen: true,
    mem_type: "video",
    parent: null,
};
function create_bitmap(width, height) {
    debug_1.log("Creating bitmap at " + width + " x " + height + "!");
    var cv = document.createElement("canvas");
    cv.width = width;
    cv.height = height;
    var ctx = cv.getContext("2d");
    if (!ctx) {
        throw new Error("Could not get context");
    }
    ctx.imageSmoothingEnabled = false;
    return {
        w: width,
        h: height,
        canvas: cv,
        context: ctx,
        ready: true,
        type: "bmp",
        clipping: true,
        clipping_rect: {
            x1: 0,
            y1: 0,
            x2: width,
            y2: height,
        },
        is_screen: false,
        mem_type: "memory",
        parent: null,
    };
}
exports.create_bitmap = create_bitmap;
function create_bitmap_ex(color_depth, width, height) {
    void color_depth;
    return create_bitmap(width, height);
}
exports.create_bitmap_ex = create_bitmap_ex;
function create_sub_bitmap(parent, x, y, width, height) {
    if (!parent) {
        return null;
    }
    var new_bmp = create_bitmap(width, height);
    sprites_1.blit(new_bmp, parent, x, y, 0, 0, width, height);
    new_bmp.parent = parent;
    return new_bmp;
}
exports.create_sub_bitmap = create_sub_bitmap;
function create_video_bitmap(width, height) {
    var bmp = create_bitmap(width, height);
    bmp.mem_type = "video";
    return bmp;
}
exports.create_video_bitmap = create_video_bitmap;
function create_system_bitmap(width, height) {
    var bmp = create_bitmap(width, height);
    bmp.mem_type = "system";
    return bmp;
}
exports.create_system_bitmap = create_system_bitmap;
function destroy_bitmap(bitmap) {
    if (!bitmap) {
        return;
    }
    bitmap.canvas.remove();
    bitmap.ready = false;
}
exports.destroy_bitmap = destroy_bitmap;
function lock_bitmap(bitmap) {
    void bitmap;
}
exports.lock_bitmap = lock_bitmap;
function bitmap_color_depth(bmp) {
    void bmp;
    return 32;
}
exports.bitmap_color_depth = bitmap_color_depth;
function bitmap_mask_color(bmp) {
    void bmp;
    return color_1.makecol(255, 0, 255);
}
exports.bitmap_mask_color = bitmap_mask_color;
function is_same_bitmap(bmp1, bmp2) {
    if (!bmp1 || !bmp2) {
        return false;
    }
    return (bmp1 === bmp2 ||
        (typeof bmp1.parent !== "undefined" && bmp1.parent === bmp2.parent));
}
exports.is_same_bitmap = is_same_bitmap;
function is_planar_bitmap(bmp) {
    void bmp;
    return false;
}
exports.is_planar_bitmap = is_planar_bitmap;
function is_linear_bitmap(bmp) {
    void bmp;
    return true;
}
exports.is_linear_bitmap = is_linear_bitmap;
function is_memory_bitmap(bmp) {
    if (!bmp) {
        return false;
    }
    return !bmp.is_screen;
}
exports.is_memory_bitmap = is_memory_bitmap;
function is_screen_bitmap(bmp) {
    if (!bmp) {
        return false;
    }
    return bmp.is_screen;
}
exports.is_screen_bitmap = is_screen_bitmap;
function is_video_bitmap(bmp) {
    if (!bmp) {
        return false;
    }
    return bmp.mem_type === "video";
}
exports.is_video_bitmap = is_video_bitmap;
function is_system_bitmap(bmp) {
    if (!bmp) {
        return false;
    }
    return bmp.mem_type === "system";
}
exports.is_system_bitmap = is_system_bitmap;
function is_sub_bitmap(bmp) {
    if (!bmp) {
        return false;
    }
    return Boolean(bmp.parent);
}
exports.is_sub_bitmap = is_sub_bitmap;
function acquire_bitmap(bmp) {
    void bmp;
}
exports.acquire_bitmap = acquire_bitmap;
function release_bitmap(bmp) {
    void bmp;
}
exports.release_bitmap = release_bitmap;
function acquire_screen() {
    acquire_bitmap(exports.screen);
}
exports.acquire_screen = acquire_screen;
function release_screen() {
    release_bitmap(exports.screen);
}
exports.release_screen = release_screen;
function set_clip_rect(bitmap, x1, y1, x2, y2) {
    if (!bitmap) {
        return;
    }
    bitmap.clipping_rect = {
        x1: x1,
        y1: y1,
        x2: x2,
        y2: y2,
    };
}
exports.set_clip_rect = set_clip_rect;
function get_clip_rect(bitmap) {
    if (!bitmap) {
        return null;
    }
    return bitmap.clipping_rect;
}
exports.get_clip_rect = get_clip_rect;
function add_clip_rect(bitmap, x1, y1, x2, y2) {
    if (!bitmap) {
        return;
    }
    bitmap.clipping_rect = {
        x1: x1,
        y1: y1,
        x2: x2,
        y2: y2,
    };
}
exports.add_clip_rect = add_clip_rect;
function set_clip_state(bitmap, state) {
    if (!bitmap) {
        return;
    }
    bitmap.clipping = state;
}
exports.set_clip_state = set_clip_state;
function get_clip_state(bitmap) {
    if (!bitmap) {
        return 0;
    }
    return bitmap.clipping ? 1 : 0;
}
exports.get_clip_state = get_clip_state;
function is_inside_bitmap(bmp, x, y, clip) {
    if (!bmp) {
        return false;
    }
    if (clip === 0) {
        return x >= 0 && y >= 0 && x < bmp.w && y < bmp.h;
    }
    return (!bmp.clipping ||
        (x >= bmp.clipping_rect.x1 &&
            y >= bmp.clipping_rect.y1 &&
            x < bmp.clipping_rect.x2 &&
            y < bmp.clipping_rect.x2));
}
exports.is_inside_bitmap = is_inside_bitmap;
function load_bitmap(filename, pal) {
    void pal;
    debug_1.log("Loading bitmap " + filename + "...");
    var img = new Image();
    img.src = filename;
    var cv = document.createElement("canvas");
    var ctx = cv.getContext("2d");
    if (!ctx) {
        throw new Error("Context not defined");
    }
    ctx.imageSmoothingEnabled = false;
    var bmp = {
        canvas: cv,
        context: ctx,
        w: -1,
        h: -1,
        ready: false,
        type: "bmp",
        clipping: true,
        clipping_rect: { x1: 0, y1: 0, x2: 0, y2: 0 },
        is_screen: false,
        mem_type: "memory",
        parent: null,
    };
    core_1._downloadables.push(bmp);
    img.onload = function () {
        debug_1.log("Bitmap " + filename + " loaded, size: " + img.width + " x " + img.height + "!");
        bmp.canvas.width = img.width;
        bmp.canvas.height = img.height;
        bmp.context.drawImage(img, 0, 0);
        bmp.w = img.width;
        bmp.h = img.height;
        bmp.clipping_rect = {
            x1: 0,
            y1: 0,
            x2: bmp.w,
            y2: bmp.h,
        };
        var imageData = bmp.context.getImageData(0, 0, bmp.canvas.width, bmp.canvas.height);
        for (var i = 0; i < imageData.data.length; i += 4) {
            if (imageData.data[i] === 255 &&
                imageData.data[i + 1] === 0 &&
                imageData.data[i + 2] === 255 &&
                imageData.data[i + 3] === 255) {
                imageData.data[i + 3] = 0;
            }
        }
        ctx.putImageData(imageData, 0, 0);
        bmp.ready = true;
    };
    return bmp;
}
exports.load_bitmap = load_bitmap;
function load_bmp(filename) {
    return load_bitmap(filename);
}
exports.load_bmp = load_bmp;
function load_bmp_pf(f, pal) {
    void f;
    void pal;
}
exports.load_bmp_pf = load_bmp_pf;
function load_lbm(filename, pal) {
    return load_bitmap(filename, pal);
}
exports.load_lbm = load_lbm;
function load_pcx(filename, pal) {
    return load_bitmap(filename, pal);
}
exports.load_pcx = load_pcx;
function load_pcx_pf(f, pal) {
    void f;
    void pal;
}
exports.load_pcx_pf = load_pcx_pf;
function load_tga(filename, pal) {
    return load_bitmap(filename, pal);
}
exports.load_tga = load_tga;
function load_tga_pf(f, pal) {
    void f;
    void pal;
}
exports.load_tga_pf = load_tga_pf;
function save_bitmap(filename, bmp, pal) {
    if (!bmp) {
        return;
    }
    void pal;
    var image = bmp.canvas.toDataURL("image/png", 1.0);
    var a = document.createElement("a");
    a.href = image;
    a.download = filename;
    document.body.appendChild(a);
    a.click();
}
exports.save_bitmap = save_bitmap;
function save_bmp(filename, bmp, pal) {
    save_bitmap(filename, bmp, pal);
}
exports.save_bmp = save_bmp;
function save_bmp_pf(f, bmp, pal) {
    void f;
    void bmp;
    void pal;
}
exports.save_bmp_pf = save_bmp_pf;
function save_pcx(filename, bmp, pal) {
    save_bitmap(filename, bmp, pal);
}
exports.save_pcx = save_pcx;
function save_pcx_pf(f, bmp, pal) {
    void f;
    void bmp;
    void pal;
}
exports.save_pcx_pf = save_pcx_pf;
function save_tga(filename, bmp, pal) {
    save_bitmap(filename, bmp, pal);
}
exports.save_tga = save_tga;
function save_tga_pf(f, bmp, pal) {
    void f;
    void bmp;
    void pal;
}
exports.save_tga_pf = save_tga_pf;
function register_bitmap_file_type(ext, load, save) {
    void ext;
    void load;
    void save;
}
exports.register_bitmap_file_type = register_bitmap_file_type;
function set_color_conversion(mode) {
    void mode;
}
exports.set_color_conversion = set_color_conversion;
function loadpng_init() {
    return 1;
}
exports.loadpng_init = loadpng_init;
//# sourceMappingURL=bitmap.js.map