interface MidiTrackEvent {
    time: number;
    type: number;
    data: number[];
    channel: number;
}
interface SMPTEOffset {
    hr: number;
    mn: number;
    se: number;
    fr: number;
    ff: number;
    time: number;
}
interface TimeSignature {
    nn: number;
    dd: number;
    cc: number;
    bb: number;
    time: number;
}
interface KeySignature {
    sf: number;
    mi: number;
    time: number;
}
interface Tempo {
    tempo: number;
    time: number;
}
interface MetaText {
    time: number;
    text: string;
}
interface NoteEvent {
    time: number;
    on: boolean;
    note: number;
    velocity: number;
    channel: number;
}
interface ProgramChange {
    time: number;
    program: number;
    channel: number;
}
interface MidiTrack {
    events: MidiTrackEvent[];
    text: MetaText[];
    copyright: MetaText[];
    trackName: MetaText[];
    instrumentName: MetaText[];
    programName: MetaText[];
    lyric: MetaText[];
    marker: MetaText[];
    cuePoint: MetaText[];
    tempo: Tempo[];
    smpteOffset: SMPTEOffset[];
    timeSignature: TimeSignature[];
    keySignature: KeySignature[];
    notes: NoteEvent[];
    programChange: ProgramChange[];
}
export interface MidiData {
    formatType: number;
    tracks: number;
    track: MidiTrack[];
    timeDivision: number[] | number;
}
export declare const parseMidi: (buffer: ArrayBuffer) => MidiData | null;
export {};
