"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports._touchmove = exports._touchend = exports._touchstart = exports._get_touch = exports._touch_loop = exports.remove_touch = exports.install_touch = exports.touch_released = exports.touch_pressed = exports.touch = exports._touch_installed = void 0;
var debug_1 = require("./debug");
var bitmap_1 = require("./bitmap");
exports._touch_installed = false;
exports.touch = [];
exports.touch_pressed = [];
exports.touch_released = [];
function install_touch() {
    if (exports._touch_installed) {
        debug_1._allog("Touch already installed");
        return -1;
    }
    bitmap_1.screen.canvas.addEventListener("touchstart", _touchstart);
    bitmap_1.screen.canvas.addEventListener("touchend", _touchend);
    bitmap_1.screen.canvas.addEventListener("touchcancel", _touchend);
    bitmap_1.screen.canvas.addEventListener("touchmove", _touchmove);
    exports._touch_installed = true;
    debug_1.log("Touch installed!");
    return 0;
}
exports.install_touch = install_touch;
function remove_touch() {
    if (!exports._touch_installed) {
        debug_1._allog("Touch not installed");
        return -1;
    }
    bitmap_1.screen.canvas.removeEventListener("touchstart", _touchstart);
    bitmap_1.screen.canvas.removeEventListener("touchend", _touchend);
    bitmap_1.screen.canvas.removeEventListener("touchcancel", _touchend);
    bitmap_1.screen.canvas.removeEventListener("touchmove", _touchmove);
    exports._touch_installed = false;
    debug_1.log("Touch removed!");
    return 0;
}
exports.remove_touch = remove_touch;
function _touch_loop() {
    if (exports._touch_installed) {
        exports.touch_released.length = 0;
        exports.touch_pressed.length = 0;
        exports.touch.forEach(function (t) {
            t.mx = 0;
            t.my = 0;
            t.px = t.x;
            t.py = t.y;
            t.age += 1;
        });
    }
}
exports._touch_loop = _touch_loop;
function _get_touch(id) {
    var _a;
    if (typeof id !== "number") {
        return null;
    }
    return (_a = exports.touch.find(function (t) { return t.id === id; })) !== null && _a !== void 0 ? _a : null;
}
exports._get_touch = _get_touch;
function _touchstart(e) {
    if (!e.target) {
        return;
    }
    var rect = e.target.getBoundingClientRect();
    for (var c = 0; c < e.changedTouches.length; c += 1) {
        var point = e.changedTouches.item(c);
        if (point) {
            var t = {
                sx: point.clientX - rect.left,
                sy: point.clientY - rect.top,
                mx: 0,
                my: 0,
                px: point.clientX - rect.left,
                py: point.clientY - rect.top,
                x: point.clientX - rect.left,
                y: point.clientY - rect.top,
                id: point.identifier,
                dead: false,
                age: 0,
            };
            exports.touch.push(t);
            exports.touch_pressed.push(t);
        }
    }
    e.preventDefault();
}
exports._touchstart = _touchstart;
function _touchend(e) {
    for (var c = 0; c < e.changedTouches.length; c += 1) {
        var point = e.changedTouches.item(c);
        var t = _get_touch(point === null || point === void 0 ? void 0 : point.identifier);
        if (t) {
            exports.touch.splice(exports.touch.indexOf(t), 1);
            exports.touch_released.push(t);
            t.dead = true;
        }
    }
    e.preventDefault();
}
exports._touchend = _touchend;
function _touchmove(e) {
    if (!e.target) {
        return;
    }
    var rect = e.target.getBoundingClientRect();
    for (var c = 0; c < e.changedTouches.length; c += 1) {
        var point = e.changedTouches.item(c);
        var t = _get_touch(point === null || point === void 0 ? void 0 : point.identifier);
        if (t && point) {
            var x = point.clientX - rect.left;
            var y = point.clientY - rect.top;
            t.mx += t.x - x;
            t.my += t.y - y;
            t.x = x;
            t.y = y;
        }
    }
    e.preventDefault();
}
exports._touchmove = _touchmove;
//# sourceMappingURL=touch.js.map