import { MidiData } from "./midiParser";
export declare type CONFIG_DATA = Record<string, Record<string, string>>;
export interface CONFIG {
    file: string;
    data: CONFIG_DATA;
    ready: boolean;
    type: "config";
}
export interface TIMER {
    timer: () => void;
    id: number;
}
export interface ALLEGRO_TOUCH_EVENT {
    sx: number;
    sy: number;
    mx: number;
    my: number;
    px: number;
    py: number;
    x: number;
    y: number;
    id: number;
    dead: boolean;
    age: number;
}
export declare type fixed = number;
export interface BITMAP {
    w: number;
    h: number;
    canvas: HTMLCanvasElement;
    context: CanvasRenderingContext2D;
    ready: boolean;
    type: "bmp";
    clipping_rect: CLIPPING_RECTANGLE;
    clipping: boolean;
    is_screen: boolean;
    mem_type: "memory" | "system" | "video";
    parent: BITMAP | null;
}
export interface CLIPPING_RECTANGLE {
    x1: number;
    y1: number;
    x2: number;
    y2: number;
}
export interface RLE_SPRITE {
    w: number;
    h: number;
    color_depth: number;
}
export interface COMPILED_SPRITE {
    w: number;
    h: number;
    color_depth: number;
    planar: number;
}
export interface JOYSTICK_INFO {
    flags: number;
    num_sticks: number;
    num_buttons: number;
    stick: JOYSTICK_STICK_INFO[];
    button: JOYSTICK_STICK_INFO[];
}
export interface JOYSTICK_BUTTON_INFO {
    b: number;
    name: string;
}
export interface JOYSTICK_STICK_INFO {
    flags: number;
    num_axis: number;
    axis: JOYSTICK_AXIS_INFO[];
    name: string;
}
export interface JOYSTICK_AXIS_INFO {
    pos: number;
    d1: number;
    d2: number;
    name: string;
}
export interface GFX_MODE_LIST {
    num_modes: number;
    mode: GFX_MODE;
}
export interface GFX_MODE {
    width: number;
    height: number;
    bpp: number;
}
export declare const PAL_SIZE = 256;
export declare type PALETTE = RGB[];
export interface RGB {
    r: number;
    g: number;
    b: number;
}
export interface V3D {
    x: number;
    y: number;
    z: number;
    u: number;
    v: number;
    c: number;
}
export interface V3D_f {
    x: number;
    y: number;
    z: number;
    u: number;
    v: number;
    c: number;
}
export interface COLOR_MAP {
    data: number[][];
}
export interface RGB_MAP {
    data: number[][][];
}
export interface al_ffblk {
    attrib: number;
    time: number;
    name: string;
}
export interface DATAFILE {
    dat: string;
    type: number;
    size: number;
    prop: string;
}
export interface MATRIX {
    v: number[][];
    t: number[];
}
export interface MATRIX_f {
    v: number[][];
    t: number[];
}
export interface QUAT {
    w: number;
    x: number;
    y: number;
    z: number;
}
export interface DIALOG {
    proc: () => void;
    x: number;
    y: number;
    w: number;
    h: number;
    fg: number;
    bg: number;
    flags: number;
    d1: number;
    d2: number;
    dp: string;
    dp2: string;
    dp3: string;
}
export interface MENU {
    text: string;
    proc: () => void;
    child: MENU;
    flags: number;
    dp: string;
}
export interface DIALOG_PLAYER {
    obj: number;
    res: number;
    mouse_obj: number;
    focus_obj: number;
    joy_on: number;
    click_wait: number;
    mouse_ox: number;
    mouse_oy: number;
    mouse_oz: number;
    mouse_b: number;
    dialog: DIALOG;
}
export interface MENU_PLAYER {
    menu: MENU;
    bar: number;
    size: number;
    sel: number;
    x: number;
    y: number;
    w: number;
    h: number;
    numberproc: () => void;
    saved: BITMAP;
    mouse_button_was_pressed: number;
    back_from_child: number;
    timestamp: number;
    mouse_sel: number;
    redraw: number;
    auto_open: number;
    ret: number;
    dialog: DIALOG;
    parent: MENU_PLAYER;
    child: MENU_PLAYER;
}
export interface FONT {
    element: HTMLStyleElement | null;
    file: string;
    name: string;
    size: number;
    type: "fnt";
}
export declare type ZBUFFER = BITMAP;
export interface SAMPLE {
    file: string;
    source: AudioBufferSourceNode;
    gain: GainNode;
    buffer: AudioBuffer | null;
    pan: StereoPannerNode;
    ready: boolean;
    type: "snd";
}
export interface MIDI {
    file: string;
    ready: boolean;
    type: "midi";
    data: MidiData;
}
export interface AUDIOSTREAM {
    voice: number;
}
export interface PACKFILE {
    vtable: PACKFILE_VTABLE;
    userdata: () => void;
    is_normal_packfile: boolean;
    normal?: {
        hndl: number;
        flags: number;
        buf_pos: number;
        buf_size: number;
        todo: number;
        parent: PACKFILE;
        pack_data: LZSS_PACK_DATA;
        unpack_data: LZSS_UNPACK_DATA;
        filename: string;
        passdata: string;
        passpos: string;
        buff: string;
    };
}
export interface PACKFILE_VTABLE {
    userdata: string;
}
export interface LZSS_PACK_DATA {
    state: number;
    i: number;
    c: number;
    len: number;
    r: number;
    s: number;
    last_match_length: number;
    code_buf_ptr: number;
    mask: number;
    code_buf: string;
    match_position: number;
    match_length: number;
    lson: number[];
    rson: number[];
    dad: number[];
    text_buf: string;
}
export interface LZSS_UNPACK_DATA {
    state: number;
    i: number;
    j: number;
    k: number;
    r: number;
    c: number;
    flags: number;
    text_buf: string;
}
